;***********************************************************************
;
;	DCF-library for the DCF77-Modul
;
;	distrubied by www.pollin.de for low cost of 4,95
;
;	Author: Peter Rebesky
;
;	Date:	20.01.2013
;
;***********************************************************************
;
;	tested on ATtiny2313 and ATmega8 with DCF77-Modul von www.pollin.de
;	fclock = 1MHz -> S_COUNT = 100 and H_L_DECIDE = 52, prescaler = 64
;	fclock = 8MHz -> S_COUNT = 200 and H_LDECIDE = 104, prescaler = 256
;
;
;
;***********************************************************************
;
;	Definitions are extern
;
;***********************************************************************
;	prepairing timer0 for the DCF-syncronisation
InitTC0:
		cli								;stop interupt
;		ldi	tempInt, 0b00000101		;set prescaler (now /1024)
		ldi	tempInt, 0b00000100		;set prescaler (now /256)
;		ldi	tempInt, 0b00000011		;set prescaler (now /64) 
;		ldi	tempInt, 0b00000010		;set prescaler (now /8)
;		ldi	tempInt, 0b00000001		;no prescaler
		out	TCCR0,	 tempInt			;and write to timer
		in	tempInt, TIMSK				;read the configuration
		ori	tempInt, 0b00000001			;add the bit of interrupt by overflow
		out	TIMSK,	 tempInt			;and write back to timer

		clr	DCFcount
		clr DCFsecond
		reti							;allow interupt
;-----------------------------------------------------------------------
;
;	The overflow-interupt of Timer0 calls this funktion ca. 60 times per second.
;	it will destroy the register r23 tempInt
;
;-----------------------------------------------------------------------
;	interupt to check the time of low DCF-signal
getDCF:
		in	saveSREG2, SREG				;save the status-register
		
		sei								;allow the interupt for TC0 -> intern clock timer, that it doesn't count wrong time	

		sbis DCF_PORT,DCF_PIN			;skip next if DCF high (second)
;		sbic PORTB,0	;DEBUG_TEST	
		inc DCFcount					;count the low time by every interupt until the DCF is high again (if high->skip this command)

		sbic DCF_PORT,DCF_PIN			;skip next if DCF low (pause between seconds)
;		sbis PORTB,0	;DEBUG_TEST
		rjmp _DCF1						;the DCF is high and jump to _DCF1
		rjmp _DCF_END					;ready and go out

_DCF1:									;DCF is on pause
		tst DCFcount					;on the first time of DCF low after high, is the DCFcount = 0. the next call is DCFcount >=1
		breq _DCF_S0					;jump out of interupt
		mov DCFcountMax, DCFcount		;save the pausen-time for STORE of high or low impulse. if short pause then high and a long pause is low impulse
		inc DCFsecond					;start and count the DCF77 seconds, it's in use to indicate the bit-positions for the time information
		set								;set the T-flag to indicate of a new start of second. Is necessary for STORE the information of time
		cpi DCFcount,S_COUNT			;sehr Frequenzabhngig vom Aufrufes des Interuptes, beim Neustart der Minute zhlt die Routine bis ber 200 bei 8Mhz
		brcs _DCF_S0					;ansonsten steht der Zhler (DCFcount) zwischen 97 und 112. Der Unterschied zeigt die 0/1 Kodierung an und wird zum dekodieren genutzt.
		clr DCFcount
		mov tempInt, DCFsecond
		clr DCFsecond					;start minute
		cbr flags,1
		cpi tempInt, 59					;check for 59 received bits. if it's not equal than is the time wrong. check the next minute
		breq _NO_ERROR
		sbr flags,0b00000001

_NO_ERROR:
		sbrc flags,0					;skip if ERROR-Bit clear
		rjmp _DCF2

		clr S_second					;set the second of intern clock to zero

		clr receiveByte
		rjmp _DCF2
_DCF_S0:
		clr DCFcount
_DCF2:
		brtc _DCF_END					;if the T-flag set than STORE the information, else go out of interupt
		mov	tempInt, DCFsecond
		cpi tempInt, 16+1				;begin when bit (16) is appeared and set tempInt at start
		brcs _DCF_END				
		dec tempInt						;the count is one time ahead
		rcall StoreDCFtime				;save the bits and bytes by decide low / high by wait-time 

_DCF_END:
		out	SREG, saveSREG2				;restore the status-register
		reti

;-----------------------------------
;	separate the time-information and store it into RAM
;	if the parity of minute and hour are false than set bit0 in flags-register to indicate an error
;	the same will be set to indicate an ERROR if the counted seconds unequal 59
;
;	this funktion will be called only once a time per second. if it more than will be all information wrong
;	to separate that is once call a second is used the t-flag!
;
StoreDCFtime:
		cpi tempInt,20					;
		brcc _STORE_MINUTE
		cpi DCFcountMax,H_L_DECIDE
		ror receiveByte
		cpi tempInt,19
		brne _END1
		sts DCFmez, receiveByte
		ret

_STORE_MINUTE:
		cpi tempInt,20
		breq _END1						;ignore the 20th bit (Minutenstartbit)
		cpi tempInt,29
		brcc _STORE_HOUR				;if bit 29 than go to store hour
		cpi DCFcountMax,H_L_DECIDE		;if received a high-bit than set carry flag (if the wait-time x < H_L_DECIDE than is high-bit and H_L_DECIDE < x than low-bit)
		ror receiveByte					;and shift into byte
		cpi tempInt,28					;shift it eight times until bit 28 appear 
		brne _END1						;all of the eight bits of minute are now store in the register "receiveByte"
		mov tempInt,receiveByte			;keep the 7th bit for it information of parity
		andi tempInt,0x7f				;delete the 7th bit (parity) but keep it for compare after count the received parity (bit 0-7)
		sts DCFminute,tempInt			;store the minute in RAM
 		clr tempInt						;and go on to count the parity
		eor tempInt,receiveByte			;first, test of parity seventh times
		lsr receiveByte
		eor tempInt,receiveByte			;second, avoid extra use of reg for a if-loop
		lsr receiveByte
		eor tempInt,receiveByte			;thirth
		lsr receiveByte
		eor tempInt,receiveByte			;fourth
		lsr receiveByte
		eor tempInt,receiveByte			;fiveth
		lsr receiveByte
		eor tempInt,receiveByte			;sixth
		lsr receiveByte
		eor tempInt,receiveByte			;seventh, result is in the LSB -> tested and it's working
		lsr receiveByte
		eor tempInt,receiveByte			;if both (DCF-parity and result-parity) identical than is result zero
		lsr tempInt						;check of zero and shift the result into carry-flag
		brcc _END1						;if high -> than ERROR else low is OK
		sbr	flags,0b00000001			;ERROR and set bit0 in flag-register
		ret

_STORE_HOUR:
		cpi tempInt,36					;if the bit 36 appear
		brcc _STORE_DAY					;than jump to the next
		cpi DCFcountMax,H_L_DECIDE				;again decide of high or low
		ror receiveByte					;and shift the result into byte
		cpi tempInt,35
		brne _END1						;only the case of check the parity of hours, else go out
		lsr receiveByte					;the same procedure like STORE_MINUTE
		mov tempInt, receiveByte
		andi tempInt,0x3f
		sts DCFhour, tempInt
 		clr tempInt						;the same parity procedure like STORE_MINUTE
		eor tempInt,receiveByte			;first, test of parity seventh times
		lsr receiveByte
		eor tempInt,receiveByte			;second, avoid extra use of reg for a if-loop
		lsr receiveByte
		eor tempInt,receiveByte			;thirth
		lsr receiveByte
		eor tempInt,receiveByte			;fourth
		lsr receiveByte
		eor tempInt,receiveByte			;fiveth
		lsr receiveByte
		eor tempInt,receiveByte			;sixth, result is in the LSB -> tested and it's working
		lsr receiveByte
		eor tempInt,receiveByte			;if both (DCF-parity and result-parity) identical than is result zero
		lsr tempInt						;check of zero and shift the result into carry-flag
		brcc _END1						;if high -> than ERROR else low is OK
		sbr	flags,0b00000001			;ERROR and set bit0 in flag-register
_END1:									;is a common return for some branch-jumps
		ret

_STORE_DAY:
		cpi tempInt,42					;again... again... again
		brcc _STORE_WEEKDAY
		cpi DCFcountMax,H_L_DECIDE
		ror receiveByte
		cpi tempInt,41
		brne _END1
		lsr receiveByte
		lsr receiveByte
		sts DCFday, receiveByte
		ret

_STORE_WEEKDAY:
		cpi tempInt,45					;again... again... again
		brcc _STORE_MONTH
		cpi DCFcountMax,H_L_DECIDE
		ror receiveByte
		cpi tempInt,44
		brne _END1
		lsr receiveByte
		swap receiveByte
		ldi tempInt,0x07
		and receiveByte,tempInt
		sts DCFweekday, receiveByte
		ret

_STORE_MONTH:
		cpi tempInt,50					;again... again... again	
		brcc _STORE_YEAR
		cpi DCFcountMax,H_L_DECIDE
		ror receiveByte
		cpi tempInt,49
		brne _END1
		lsr receiveByte
		lsr receiveByte
		lsr receiveByte
		sts DCFmonth, receiveByte
		ret

_STORE_YEAR:
		cpi tempInt,58					;again... again... again
		brcc _END1
		cpi DCFcountMax,H_L_DECIDE
		ror receiveByte
		cpi tempInt,57
		brne _END1
		sts DCFyear, receiveByte
		ret


;--- END ---
