;***********************************
;
;	Projekt Gardienensteuerung mit DCF77 und LCD-Display Version 1.0
;	Author:	Peter Rebesky
;
;	06.03.2013
;
;***********************************
;
.DEVICE ATmega8
.NOLIST
.INCLUDE "m8def.inc"
;.INCLUDE "Lochraster_ATmega8.inc"
.INCLUDE "GardineSMD.inc"
.LIST
;
;-----------------------------------

.equ	MenuItem		= 9	;max counts of menu

;-----------------------------------
; register definitions
;

.def 	receiveByte		= r1	;byte for received byte from DCF77	
.def 	OldMinute		= r2	;is for used to indicate if a new minute arrived. This variable saved the old minute and when increment S_minute then there is a difference.
.def 	S_hour			= r3	;hour of intern clock
.def 	S_minute		= r4	;minute of intern clock
.def 	S_day			= r5	;day of intern clock
.def 	variable		= r6
.def 	MenuCount		= r7	;counter of selected menu, it will be automatic reseted after xx seconds
.def 	MenuSetCount	= r8	;counter of selected Set-Menu, it will be reseted after xx seconds automaticly
.def 	LCD_C_Point 	= r9	;LCD point of character counter
.def 	value			= r10	;is in use by interupt (intern clock for return-value of days per month)
.def 	DCFsecond		= r11	;DCF77 second-counter
.def 	OldSecond		= r12
.def 	D_Info			= r13
.def 	saveSREG1		= r14	;is in use by interupt TC0, very important to save the status-register
.def 	saveSREG2		= r15	;is in use by interupt TC1, very important to save the status-register
.def 	temp1 			= r16	;temporres Register zum Laden von Variablen
.def 	temp2 			= r17	;temporres Register
.def 	data 			= r18	;Daten zum/aus EEprom
.def 	BitCount 		= r19 	;register fr Zugriff auf EEprom
.def 	DCFcount 		= r20	;count the calls of interupt (overflow of counter0) meanwhile is the signal low
.def 	DCFcountMax 	= r21	;is in used to indicate if DCF received high or low
.def 	flags			= r22	;bit0: DCF-ERROR
								;bit1:Button right, is in use for debouncing
								;bit2:Button left, is in use for debouncing
								;bit3:courtains close ->high, is in use to indicate of courtains are close and switch light on
								;bit4:
								;bit5:leapyear => high, else = low
								;bit6:turn automatic off, no times will be set courtain or light
								;bit7:startmarker (if set 0 when clock is syncron the first time)
.def 	tempInt			= r23	;is in use by interupt
.def 	S_second		= r24	;is in use by interupt
.def 	count			= r25	;is in use by interupt
.def 	rCMD_low		= r26	;XL
.def 	rCMD_High 		= r27	;XH
;.def 	XH				= r27	;RAMhigh
;.def 	XL				= r26	;RAMlow
;.def 	YH				= r29
;.def 	YL				= r28
;.def 	ZH				= r31	
;.def 	ZL				= r30	

;-----------------------------------
;	Debug Values
.equ Char = ' '

;-----------------------------------
; sram definations, variables
;
.DSEG
.ORG SRAM_START

DCF77Info:
DCFmez:		.db 0	;DCF MEZ / MESZ
DCFminute:	.db 0	;DCFminute
DCFhour:	.db 0	;DCFhour
DCFday:		.db 0	;DCFday
DCFweekday:	.db 0	;DCFweekday
DCFmonth:	.db 0	;DCFmonth
DCFyear:	.db 0	;DCFyear
;************************************
D_DATE:
D_mez:		.db 0	;DisplayMEZ / MESZ
D_weekday:	.db 0	;Displayweekday
D_month:	.db 0	;Displaymonth
D_year:		.db 0	;Displayyear

O_hour:		.db 0	;hour open-time
O_minute:	.db 0	;minute open-time
C_hour:		.db 0	;hour close-time
C_minute:	.db 0	;minute close-time

L_hour:		.db 0	;hour of turn off the light
L_minute:	.db 0	;minute of turn off the light

TimerFlags:	.db 0	;showed when timer is running. high of bit 0 -> run timer0
Timer0:		.db 0
Timer1:		.db 0
Timer2:		.db 0
Timer3:		.db 0
Timer4:		.db 0
Timer5:		.db 0
Timer6:		.db 0
Timer7:		.db 0
Time0:		.db 0
Time1:		.db 0
Time2:		.db 0
Time3:		.db 0
Time4:		.db 0
Time5:		.db 0
Time6:		.db 0
Time7:		.db 0

dummy:		.db 0x20;dummy for debug-information
dummy2:		.db 0
dummy3:		.db 0
dummy4:		.db 0	

;-----------------------------------
;	reset and interupt vectors
.CSEG
.ORG $0000
		rjmp 	main	; reset vector
		reti 			; external int 0
		reti			; external int 1
		reti			; Counter2 Compare
		reti			; Counter2 Overflow
		reti			; Counter1 Capture event
		rjmp 	countUP	; Counter1 Compare Match A
		reti			; Counter1 Compare Match B
		reti			; Counter1 overflow
		rjmp 	getDCF	; Counter0 overflow
		reti			; USART Rx Complete
		reti			; USART Data empty
		reti			; USART Tx Complete
		reti			; ADC Conversion Complete
		reti			; EEPROM Ready
		reti			; Analog Comparator
		reti			; Two-wire Serial Interface
		reti			; Store Program Memory Ready

;-----------------------------------
;	main programstart and main routine

main:
		wdr
        ldi 	temp1, (1<<WDCE)|(1<<WDE) 				;set the watchdoc, first start are 16ms
        out 	WDTCR, temp1
		ori		temp1, (1<<WDP2)|(1<<WDP1)|(0<<WDP0)	;it's important to set the watchdoc on and after it set the prescaler, waiting time before reset = 1,0 sec
		out		WDTCR, temp1							
;********** after this code the watchdoc is present and enabled ****************

		ldi 	temp1, LOW(RAMEND)
    	out 	SPL, temp1
    	ldi 	temp1, HIGH(RAMEND)
    	out 	SPH, temp1


;-----------------------------------
		ldi		temp1, 0x10
_main1:
		dec		temp1			;this waiting-time is necessary for init itself of the display (lcd) and the other hardware after turn the power on.
		push	temp1			;the lcd display need some time to oparate exactly after switch the power on
		rcall	delay_max
		pop		temp1
		tst		temp1
		brne	_main1
;-----------------------------------


		rcall 	InitLCD
		rcall	InitMainValues
		rcall 	InitValues

#ifdef _with_RFM_
		rcall	InitRFMsend
#endif
		wdr
		rcall 	InitRTC
		rcall 	InitTC0

		rcall 	LCD_1RowHome
		ldi 	XH, high(InfoString)	;show the start information after turn the power on
		ldi		XL, low(InfoString)
		rcall 	Show_String
		rcall 	LCD_Clear

		sei							;turn the interupts on, TC0 and TC1



;----------

loop:
		wdr							;reset the watchdoc
		sleep						;a short sleep for the controller to save power
		nop
		rcall 	SynchronDCF			;check out to syncron DCF77 to intern clock, it will be synchron only if no error occurs

		cp		S_minute,OldMinute	;check out when a new minute arrived
		breq	loop1				;after every minute will do in order after this commando only once a time
		rcall	CheckAction			;do the action like switch relais and go on
		rcall	CheckLeapyear		;set the bit5 in flags if a leapyear occurs 2012,2016,2020,2024... and so on

loop1:
		cp		S_second,OldSecond	;check out when a new second arrived
		breq	loop2				;only once a time per second will call the next commando
		rcall	RunTimer			;control of the softwaretimer for motor-cycle-time and other futures like the menu-reset of itselfs
		
loop2:
#ifdef _with_DCF77_
;		sbi		LEDPort,LEDblue		;LED for impulse of DCF-signal
;		sbis	DCF_PORT, DCF_PIN	;show the DCF-Impuls by LED 
;		cbi		LEDPort,LEDblue	
#endif	

		rcall	SelectMenu			;check out if button menu pressed and count up the menue-counter or menusetcounter
		rcall 	LCD_1RowHome		;set the first position on LCD
		sbrs	MenuSetCount,7		;skip next order when Set-Mode is activ
		rcall 	DisplayTime			;show the time on LCD at first line by 8 character like hh:mm:ss

		rcall 	LCD_2RowHome		;next row on the display

;******************************************************
;	case / break - routine for decision of display the menue in range of 0 til MenuItem
		mov 	temp2, MenuCount
		cpi 	temp2, MenuItem + 1	; "+1" because the C-flag is set only when argument is larger then value of register
		brcc 	_NOT_IN_MENU		;out of range (auerhalb des Bereiches)
		ldi 	ZL, low(_Menu)
		ldi 	ZH, high(_Menu)
		lsl 	temp2				;multiplied by two because there are two commandos in the jump-table
		add 	ZL,temp2			;variable addieren
		clr 	temp2
		adc 	ZH,temp2			;16 bit add include the carry-flag (important)
		ijmp
_Menu:
		rcall	DisplayDate			;case: 0
		rjmp 	_MenuEnd
		rcall	SwitchOn_Off		;case: 1
		rjmp 	_MenuEnd
		rcall	SwitchLightOnOff	;case: 2
		rjmp 	_MenuEnd
		rcall	CurtainOpen			;case: 3
		rjmp 	_MenuEnd
		rcall	CurtainClose		;case: 4
		rjmp 	_MenuEnd
		rcall	DisplayOpenTime		;case: 5
		rjmp 	_MenuEnd
		rcall	DisplayCloseTime	;case: 6
		rjmp 	_MenuEnd
		rcall	DisplayTimer		;case: 7
		rjmp 	_MenuEnd
		rcall	DisplayDebug		;case: 8
		rjmp 	_MenuEnd
		rcall	Set_Mode			;case: 9
		rjmp 	_MenuEnd
_NOT_IN_MENU: 						;case: default (ERROR)
		ldi 	XH,high(StringError)
		ldi		XL,low(StringError)
		rcall 	Show_String

_MenuEnd:					;break
		rjmp	loop

;***********************************************************
;
;	Displayed the status of light if on or off in row two of LCD
;	even change it
;			
SwitchLightOnOff:

		ldi		XH,high(StringLo)
		ldi		XL,low(StringLo)
		sbrs	flags, 3					;if turn automatic off
		ldi 	XL,low(StringLf)			; !!! important that XH not change itself !!! it must be placed together in the first 255 bytes into the eeprom!
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		rjmp	_SwitchLightOnOFF
		sbr		flags,4
		ret
_SwitchLightOnOFF:
		sbrs	flags,2						;button debouncing
		ret
		cbr		flags,4
		
		sbrs	flags,3						;exor with bit 6
		rjmp	_SetLightOnOff
		cbr		flags,8
		ret
_SetLightOnOff:
		sbr		flags,8
		ret
;--- end ---

;***********************************************************
;
;	Close courtains?
;			
CurtainClose:
		ldi		XH,high(StringClose)
		ldi 	XL,low(StringClose)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		ret

		lds		temp1, TimerFlags			;start timer for draw the cortains
		ori		temp1, 0b00000001			;set flag of start timer0
		sts		TimerFlags, temp1
		sbr		flags,8						;flag if courtains open?
		sbi		RelaisPort, RelaisDirection
		sbi		RelaisPort, RelaisUb		;turn on the Ub

;		sbi		LEDPort, LEDyellow			;show on yellow LED

		ret

;--- end ---
;***********************************************************
;
;	Open courtains?
;			
CurtainOpen:
		ldi		XH,high(StringOpen)
		ldi 	XL,low(StringOpen)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		ret

		lds		temp1, TimerFlags			;start timer for draw the cortains
		ori		temp1, 0b00000010			;set flag of start timer1
		sts		TimerFlags, temp1
		cbr		flags,8						;flag if courtains open?
		sbi		RelaisPort,RelaisUb

;		sbi		LEDPort, LEDred		;show on red LED

		ret

;--- end ---

;***********************************************************
;
;	clear 65 bytes of SRAM from SRAM_START
;	and load the defaults
;	
InitMainValues:	
		ldi		temp1, 64
		ldi		temp2, 0
		ldi		XH, high (SRAM_START)
		ldi		XL, low (SRAM_START)
_InitMainV:
		st		X, temp2			;write to used SRAM zero
		inc		XL
		subi	temp1,1
		brcc	_InitMainV

		out 	eearh, temp2		;set the high address bit8 of EEPROM on zero -> maximal 256 byte are used in some cases
		clr		flags
	
		sbi		RelaisDDR, RelaisUb
		sbi		RelaisDDR, RelaisDirection

		sbi		LED_DDR,LEDyellow
		sbi		LED_DDR,LEDgreen
		sbi		LED_DDR,LEDred

#ifdef _with_RFM ;********** only if RFM available *********	
		cbi		DDRD,send_SDO

		sbi		DDRD,send_nSEL
		sbi		DDRD,send_FSK
		sbi		DDRC,send_SDI
		sbi		DDRC,send_SCK

		sbi		PORTD,send_nSEL
		sbi		PORTD,send_FSK
		sbi		PORTC,send_SCK
		sbi		PORTC,send_SDI
#endif ;****************************************************

		sbi		ButtonPortOut, ButtonRight	;set pull-up resistor
		sbi		ButtonPortOut, ButtonLeft	;set pull-up resistor
		sbi		DCF_PORT_RES,DCF_PIN		;set pull-up resistor for the input pin of DCF77 (without it coult be not working!)

		sbr 	flags,1				;error indicator of desyncron DCF77 and intern clock
		sbr		flags,128			;startmarker, if necessary to indicate for wrong switch-time
		clr 	XH
		clr 	XL
		clr		OldMinute
		clr		OLDsecond
		clr		S_day
		clr		MenuCount			;initial menucount
		clr		MenuSetCount
		ldi		temp1, 0x20			;load a space-character
		mov		D_Info,temp1		;it is show between month and weekday on the display


		ldi		XH, high(CloseTime)	;close time in seconds to turn on the motor
		ldi		XL, low(CloseTime)
		out		eearh, XH
		out		eearl, XL
		sbi 	eecr,0
		in 		data, eedr			;set timer0 on eeprom sec (this are not the ctc of ATmega!!!) software ctc!
		sts		Time0, data

		ldi		XH, high(OpenTime)	;close time in seconds to turn on the motor
		ldi		XL, low(OpenTime)
		out		eearh, XH
		out		eearl, XL
		sbi 	eecr,0			
		in 		data, eedr			;set timer1 on eeprom sec (this are software-time for intern control of time)
		sts		Time1, data			;open time in seconds to turn on the motor

		ldi		XH, high(LightOff)	;turn off time for the light
		ldi		XL, low(LightOff)
		out		eearh, XH
		out		eearl, XL
		sbi 	eecr,0			
		in 		data, eedr			
		sts		L_minute, data
		adiw	XH:XL,1
		out		eearh, XH
		out		eearl, XL
		sbi 	eecr,0			
		in 		data, eedr			
		sts		L_hour, data

		ldi		temp1, 13			;load the beginning year 2013
		sts		D_year, temp1

		ldi		temp1, 120
		sts		Time2, temp1		;set timer2 on 120 sec for automatic reset the MenuCount of zero

		ldi		temp2, 90			;90 seconds for impuls to switch the LED (light) on
		sts		Time3, temp2

		ldi		temp1, 20			;load the default times 20:15
		sts		C_hour, temp1

		ldi		temp1, 15
		sts		C_minute, temp1

		ldi		temp1, 8			;and 8:00 o clock for open courtains
		sts		O_hour, temp1

		clr		temp1
		sts		O_minute, temp1
		ret

;--- end ---
;***********************************************************
;
;	send a commando per RF to all the receiver OR switch the relay on for 90 sec 
;	input:
;	output: in temp1, ok= 0, fault= -1 (0xff)	
;	this function will be called every 60 seconds
;
SendCMD_RF:	
		
#ifdef _without_RFM_

		sbi		RelaisPort, RelaisLED	;switch LED on for 90 sec and start timer 3 (software ctc)

#endif
#ifdef _with_RFM_

		rcall	RFM_tx_data			;is only used if a remoted controlled switch available
#endif
		sbi		LEDPort, LEDblue
		lds		temp2, TimerFlags		;start timer vor switching-time of relais
		ori		temp2, 0b00001000		;set flag of start timer3
		sts		TimerFlags, temp2		;and store it into SRAM
		clr		temp2
		sts		Timer3, temp2			;reset the timer3 for a new start. this will be happen every minute (60 sec)
		clr		temp1				;set temp1 on zero to indicate "no error"
		ret

;--- end ---
;***********************************************************
;
;	get the time when open or close the cover
;	input: week calculated by CheckTime, stored in temp1
;	output: temp1= 1 if open; 2 if close; -1 if Error
;	C_hour, C_minute, O_hour, O_minute -> are stored the times
;	
GetSwitchTime:	
		lds		temp2, D_month		;get actual month and test it if zero, that mean no set by DCF
		tst		temp2				;if month not set then no time from EEPROM
		breq	_GetError			;the Date is zero -> not syncron with DCF77 and error! no astro-funktion

		clr		temp2
		out		eearh,temp2 
		ldi 	XL,AstroTable
		lsl		temp1				;multiple by 2, because there are two bytes (hour and minute)
		add		XL,temp1			;in bitCount is pointer of time depending of week
		cpi		XL, AstroTable + 91 ;45 weeks multipled by 2 (minute and hour)
		brcc	_GetError

		clr		temp1
		out		eearh, temp1		;the data must be placed in the lower 255 bytes of eeprom!!!
		out 	eearl, XL			;get minute from eeprom
		sbi 	eecr,0
		in 		data, eedr
		sts		C_minute,data
		inc		XL
		out 	eearl, XL			;get hour from eeprom
		sbi 	eecr,0
		in 		data, eedr
;*** check if summertime
		lds		temp1, D_mez		;get info if summertime;	Bit1 = MESZ; Bit2 = MEZ
		sbrc	temp1,1				;if bit of summertime set?
		inc		data				;when then add 1 hour
		sts		C_hour,data
;*** get the time of open
		ldi		XL,OpenCourtain
		lds		temp1,D_weekday
		dec		temp1
		lsl		temp1
		add		XL,temp1
		cpi		XL,OpenCourtain + 13 ;if result higher than 13 (6 days multiplied by 2) error
		brcc	_GetError
		out 	eearl, XL			;get minute
		sbi 	eecr,0
		in 		data, eedr
		sts		O_minute,data		;time of open cover
		inc		XL
		out 	eearl, XL			;get hour
		sbi 	eecr,0
		in 		data, eedr
		sts		O_hour,data			;time of open cover
		ret

_GetError:
		ldi		temp1,-1
		ret

;--- end ---
;***********************************************************
;
;	once a time in a minute will call and execute this funktion	
;	
CheckAction:	

		mov		OldMinute, S_minute
		rcall	CalcWeek			;out temp1, get the the number of week (8 days) for take the astro-time
		rcall	GetSwitchTime		;in temp1, get the stored astro-time
		sbi		LEDPort, LEDred		;show if the automatic is turn off -> red LED on
		sbrc	flags, 6			;if turn automatic off then get out of here by ret
		ret

		cbi		LEDPort, LEDred		;clear if automatic is on
		rcall	SwitchRelaisClose	;if the time appeared than switch the motor for close
		rcall	SwitchRelaisOpen	;if the time appeared than switch the motor for open
		rcall	SwitchLightOn		;if the light turn on? then send a RF commando
		ret

;--- end ---
;***********************************************************
;	
;	
;	
SwitchRelaisClose:
		lds		temp2, C_hour			;get hour of close-time
;		ldi		temp2, 0x12									;it's for debug-mode only
		cp		temp2, S_hour			;compare switch-time
		brne	_SClose					;not equal
		lds		temp2, C_minute			;get minute
;		ldi		temp2, 0x2d									;it's for debug-mode only
		cp		temp2, S_minute
		brne	_SClose					;not equal
		sbi		RelaisPort, RelaisDirection		;turn on the relais for direction for close the covers
		sbi		RelaisPort, RelaisUb	;turn on the voltage for the relais
;		sbi		LEDPort, LEDyellow		;show on yellow LED
		sbr		flags,8					;flag for light when if open or close
		lds		temp2, TimerFlags		;start timer vor switching-time of relais
		ori		temp2, 0b00000001		;set flag of start timer0
		sts		TimerFlags, temp2		;and store it into SRAM
_SClose:
		ret
;--- end ---
;***********************************************************
;	
;	
;	
SwitchRelaisOpen:
		lds		temp2, O_hour			;get hour of close-time
;		ldi		temp2, 0x12									;it's for debug-mode only
		cp		temp2, S_hour			;compare switch-time
		brne	_SClose					;not equal
		lds		temp2, O_minute			;get minute
;		ldi		temp2, 0x37									;it's for debug-mode only
		cp		temp2, S_minute
		brne	_SOpen					;not equal
		cbr		flags,8					;flag for light when if open or close
;		sbi		RelaisPort, RelaisDirection		;turn on the relais for direction for close the covers
		sbi		RelaisPort, RelaisUb	;turn on the voltage for the relais
;		sbi		LEDPort, LEDred		;show on red LED
		lds		temp2, TimerFlags		;start timer vor switching-time of relais
		ori		temp2, 0b00000010		;set flag of start timer1
		sts		TimerFlags, temp2		;and store it into SRAM
_SOpen:
		ret
;--- end ---
;***********************************************************
;	
;	send the commando to turn the light on for 5 min (or 90 seconds in relay mode)
;	the remote-switch turns the light on, every one minute for 5 minute	(90 sec)
;	
SwitchLightOn:
		
		sbrs	flags,3				;skip if courtains are close
		ret							;else go out without turn the light on

		lds		temp1, L_minute
		cp		temp1, S_minute		;if time and clock identicaly?
		brne	_SwitchLightOn

		lds		temp1, L_hour
		cp		temp1, S_hour		;if time and clock identicaly?
		brne	_SwitchLightOn

		cbr		flags, 8			;clear bit for turn the light off. 8 (b00001000) -> cleared the bit3 in flags
		ret

_SwitchLightOn:
		rcall	SendCMD_RF			;turn on the light by RF-module

		ret
;--- end ---
;***********************************************************
;	
;	this function will be called every second and controlled the counter of timers	
;	
RunTimer:
		mov		OldSecond, S_second		;it's necessary for indicate that this funktion called already

		lds		data, TimerFlags
		tst		data
		breq	_RunTimerRet		;no flags are set, that mean no timer is running and get out of here

		sbrc	data,0				;case of which timer is running
		rcall	RunTimer0

		sbrc	data,1
		rcall	RunTimer1

		sbrc	data,2
		rcall	RunTimer2

		sbrc	data,3
		rcall	RunTimer3

		sbrc	data,4
		rcall	RunTimer4

		sbrc	data,5
		rcall	RunTimer5

		sbrc	data,6
		rcall	RunTimer6

		sbrc	data,7
		rcall	RunTimer7

_RunTimerRet:
		ret
;--- end ---
;***********************************************************
;	7 software-timers
;	seven timers for control of times about 1 until 255 seconds	
;	
RunTimer0:							;Gardine close time
		lds		temp1, Timer0
		lds		temp2, Time0
		inc		temp1
		sts		Timer0, temp1
		cp		temp1, temp2
		brcs	_Ret0
		clr		temp1
		sts		Timer0, temp1
		lds		temp1, TimerFlags
		andi	temp1, 0b11111110	;clear the time0 flag
		sts		TimerFlags, temp1
		; Action when timer0 ready
		cbi		RelaisPort, RelaisUb		;turn off the Ub
		cbi		RelaisPort, RelaisDirection
;		cbi		LEDPort, LEDyellow	;clear show on yellow LED

_Ret0:
		ret
;***********************************************************
RunTimer1:							;Gardine open time
		lds		temp1, Timer1
		lds		temp2, Time1
		inc		temp1
		sts		Timer1, temp1
		cp		temp1, temp2
		brcs	_Ret1
		clr		temp1
		sts		Timer1, temp1
		lds		temp1, TimerFlags
		andi	temp1, 0b11111101	;clear the time1 flag
		sts		TimerFlags, temp1
		; Action when timer1 ready
		cbi		RelaisPort, RelaisUb
;		cbi		LEDPort, LEDred		;clear show on red LED

_Ret1:
		ret
;***********************************************************
RunTimer2:							;reset menu after 60 seconds
		lds		temp1, Timer2
		lds		temp2, Time2
		inc		temp1
		sts		Timer2, temp1
		cp		temp1, temp2
		brcs	_Ret2
		clr		temp1
		sts		Timer2, temp1
		lds		temp1, TimerFlags
		andi	temp1, 0b11111011	;clear the time2 flag
		sts		TimerFlags, temp1
		; Action when timer2 ready
		clr		MenuCount
		clr		MenuSetCount
_Ret2:
		ret
;***********************************************************
RunTimer3:							;is for relay of LED
		lds		temp1, Timer3
		lds		temp2, Time3
		inc		temp1
		sts		Timer3, temp1
		cp		temp1, temp2
		brcs	_Ret3
		clr		temp1
		sts		Timer3, temp1
		lds		temp1, TimerFlags
		andi	temp1, 0b11110111	;clear the time2 flag
		sts		TimerFlags, temp1
		; Action when timer3 ready
#ifdef _without_RFM_
		cbi		RelaisPort, RelaisLED
#endif
		cbi		LEDPort, LEDblue
_Ret3:
		ret
;***********************************************************
RunTimer4:							;not in use
		lds		temp1, Timer4
		lds		temp2, Time4
		inc		temp1
		sts		Timer4, temp1
		cp		temp1, temp2
		brcs	_Ret4
		clr		temp1
		sts		Timer4, temp1
		lds		temp1, TimerFlags
		andi	temp1, 0b11101111	;clear the time2 flag
		sts		TimerFlags, temp1
		; Action when timer4 ready
_Ret4:
		ret
;***********************************************************
RunTimer5:							;not in use
		lds		temp1, Timer5
		lds		temp2, Time5
		inc		temp1
		sts		Timer5, temp1
		cp		temp1, temp2
		brcs	_Ret5
		clr		temp1
		sts		Timer5, temp1
		lds		temp1, TimerFlags
		andi	temp1, 0b11011111	;clear the time2 flag
		sts		TimerFlags, temp1
		; Action when timer5 ready
_Ret5:
		ret
;***********************************************************
RunTimer6:							;not in use
		lds		temp1, Timer6
		lds		temp2, Time6
		inc		temp1
		sts		Timer6, temp1
		cp		temp1, temp2
		brcs	_Ret6
		clr		temp1
		sts		Timer6, temp1
		lds		temp1, TimerFlags
		andi	temp1, 0b10111111	;clear the time2 flag
		sts		TimerFlags, temp1
		; Action when timer6 ready
_Ret6:
		ret
;***********************************************************
RunTimer7:							;not in use
		lds		temp1, Timer7
		lds		temp2, Time7
		inc		temp1
		sts		Timer7, temp1
		cp		temp1, temp2
		brcs	_Ret7
		clr		temp1
		sts		Timer7, temp1
		lds		temp1, TimerFlags
		andi	temp1, 0b01111111	;clear the time2 flag
		sts		TimerFlags, temp1
		; Action when timer7 ready
_Ret7:
		ret
;--- end ---

;***********************************************************	
;	calculated the weeks -> days * month-1 + days(act.month) / 8
;	the fundation of days per month are 30 days (it's easier than lock every month if 30 or 31)
;
CalcWeek:
		
		lds		temp1,D_month
		clr		temp2
		subi	temp1,1			;sub the actual month because there are the days to add
		tst		temp1			;check for zero, than go to add the days
		breq	_MULTI3
		mov		data,temp1
		dec		data
;the fast multiplication by 30 and division by 8
		lsl		temp1			;multiple by 2, 4, 8, 16, 32
		rol		temp2			;with 16 bit (two register)
		lsl		temp1
		rol		temp2
		lsl		temp1
		rol		temp2
		lsl		temp1
		rol		temp2
		lsl		temp1
		rol		temp2
_MULTI2:
		subi	temp1,2			;sub the 2 days, because multiple by 32 and 30 are necessary only
		sbci	temp2,0			;sub the carry-flag for a 16bit subtraction
		subi	data,1			
		brcc	_MULTI2			;how often sub2, like countes of month
_MULTI3:
		clr		data
		add		temp1,S_day
		adc		temp2,data
		lsr		temp2
		ror		temp1			;divided by 8 (2,4,8)
		lsr		temp2
		ror		temp1
		lsr		temp2
		ror		temp1			;now, in temp1 is the pointer of week where is coded the sun goes down-time.
_MULTI4:
		ret
;---end---
;***********************************************************
;
;	show some values on LCD for inter information
;
;this funktion don't show the timers not any more, i used to show the counters for indicate the times of DCF-signal
DisplayTimer:	
		mov		temp2,DCFcountMax	;is the value of counted low times from DCF77 impuls, the sooner the count if a long low time. this in used to indicate if high or low
		rcall 	bin2BCD
		rcall 	ShowBCD
		ldi		temp1, 35

#ifdef _withDCF77_
		sbis	DCF_PORT, DCF_PIN	;show the DCF-Impuls on LCD
		ldi		temp1, 32
#endif
		rcall 	LCD_Data			; a # on LCD shows the low time of DCF77 signal

		mov		temp2,receiveByte
		rcall 	bin2BCD
		rcall 	ShowBCD
		ldi		temp1, 32
		rcall 	LCD_Data
	
		lds		temp2,Timer2		;this value shows the time until the menu will be reseted (by 120) and shows the standard display
		rcall 	bin2BCD
		rcall 	ShowBCD
		ret
;---end---
;***********************************************************
DisplayCloseTime:		;show the time when close the courtains
		lds		temp2,C_hour
		rcall 	bin2BCD
		rcall 	ShowBCD
		ldi		temp1, 58
		rcall 	LCD_Data

		lds		temp2,C_minute
		rcall 	bin2BCD
		rcall 	ShowBCD
		ldi		temp1, 32
		rcall 	LCD_Data

		ldi		temp1, 122
		rcall 	LCD_Data
		ldi		temp1, 117
		rcall 	LCD_Data
	
		ret
;---end---

;***********************************************************
DisplayOpenTime:		;show the time when open the courtains
		lds		temp2,O_hour
		rcall 	bin2BCD
		rcall 	ShowBCD
		ldi		temp1, 58
		rcall 	LCD_Data

		lds		temp2,O_minute
		rcall 	bin2BCD
		rcall 	ShowBCD

		ldi		temp1, 97
		rcall 	LCD_Data
		ldi		temp1, 117
		rcall 	LCD_Data
		ldi		temp1, 102
		rcall 	LCD_Data
	
		ret
;---end---

;***********************************************************
DisplayDebug:			;show the register of flags for debug
;		lds		temp2,DCFmez
		ldi		temp2,0
		sbrc	flags,1
		ldi		temp2,1
		sbrc	flags,0
		sbr		temp2,16
;		rcall 	bin2BCD
		rcall 	ShowBCD
;		ldi		temp1, 0x20
;		rcall 	LCD_Data

;		lds		temp2,D_mez
		ldi		temp2,0
		sbrc	flags,3
		ldi		temp2,1
		sbrc	flags,2
		sbr		temp2,16
;		rcall 	bin2BCD
		rcall 	ShowBCD
;		ldi		temp1, 0x20
;		rcall 	LCD_Data
	
;		mov		temp2,DCFcountMax
		ldi		temp2,0
		sbrc	flags,5
		ldi		temp2,1
		sbrc	flags,4
		sbr		temp2,16
;		rcall 	bin2BCD
		rcall 	ShowBCD

		ldi		temp2,0
		sbrc	flags,7
		ldi		temp2,1
		sbrc	flags,6
		sbr		temp2,16
;		rcall 	bin2BCD
		rcall 	ShowBCD

		ret
;---end---
;***********************************************************
;	show a string on actual LCD-position pointed out by bitCount
;	input: XL = low(pointer of string), XH = high(pointer of string)
;	output: LCD display -> show string until 0 occurs
;	
Show_String:			
		out		eearh, XH		;load the address of EEPROM
		out 	eearl, XL		;the string must be stored into EEPROM (by ATtiny there is the low byte-adress only)
		sbi 	eecr,0
		in 		temp1, eedr
		tst 	temp1
		breq 	return
		rcall 	LCD_Data_Run
		adiw	XH:XL, 1
		rjmp 	Show_String
return:
		ret	
;************************************
.include	"LCD_C0802_04.inc"
.include	"DCF77Pollin.inc"
.include	"IntClock.inc"
.include	"SetMenu.inc"
.include	"RFM02S.inc"

;************************************
;
;	EEPROM Segment
;


.ESEG

Sonntag:	.db "So" .db 0
Montag:		.db "Mo" .db 0
Dienstag:	.db "Di" .db 0
Mittwoch:	.db "Mi" .db 0
Donnerstag:	.db "Do" .db 0
Freitag:	.db "Fr" .db 0
Sonnabend:	.db "Sa" .db 0
Error:		.db "ER" .db 0

LightOff:			;Licht aus um xy Uhr
.dw 0x0100			; 1:00 o clock at night

OpenTime:
.db 48
CloseTime:
.db 41

OpenCourtain:
;	Mo;		Di;		Mi;		Do;		Fr;	Sa;		So;	 Error
.dw 0x0700,0x0700,0x0700,0x0700,0x0700,0x0a1e,0x0a1e,0x0000

AstroTable:			;Sonnenuntergang pro Woche im Jahr, Hexadecimal codiert in Stunde und Minute, 140c == 20:12 Uhr :-)
.dw 0x1015,0x101d,0x1028,0x1034,0x1106,0x1114,0x1122,0x1130,0x1202,0x120e,0x121d,0x122a,0x1238,0x1309,0x1317,0x1324,0x1331,0x1401,0x140c,0x1415,0x141c,0x1421,0x1422,0x1420
.dw 0x141a,0x1412,0x1407,0x1336,0x1327,0x1317,0x1305,0x122f,0x121c,0x1209,0x1133,0x1120,0x110f,0x1039,0x1029,0x101d,0x1013,0x100d,0x1009,0x1009,0x1008,0x100b,0x1010,0x0000
	
InfoString:
	.db "Gardinensteuerung 1.0, P.Rebesky    ".db 0
StringOpen:
	.db "Open: ? " .db 0
StringClose:
	.db	"Close:? " .db 0
StringSetH:
	.db "^    Set" .db 0
StringSetM:
	.db "   ^ Set" .db 0
StringSetOT:
	.db "Open?" .db 0
StringSetCT:
	.db "Close=" .db 0
StringLo:
	.db "Light On" .db 0
StringLf:
	.db "LightOff" .db 0
StringOn:
	.db "On->Off?" .db 0
StringOff:
	.db "Off->On?" .db 0
StringSet:
	.db "SetMenu?" .db 0
StringRO:
	.db "Courtain" .db 0
StringROP:
	.db "Open= " .db 0
StringRCL:
	.db "Close=" .db 0
StringDisplayMonth:
	.db "Month=" .db 0
StringDisplayYear:
	.db "Set Year" .db 0
StringSetMD:
	.db "Set ?   " .db 0
StringDisplayDay:
	.db "Day=  " .db 0
StringDisplayWeekday:
	.db "Set Week" .db 0
StringSetWeekday:
	.db "day:  " .db 0
StringLightClock:
	.db "LED off?" .db 0
StringLightHour:
	.db "Hour: " .db 0
StringLightMinute:
	.db "Min:  " .db 0
StringMEZ:
	.db "MEZ-MESZ" .db 0
StringMESZ:
	.db "MESZ-MEZ" .db 0
StringChange:
	.db "Change ?" .db 0
StringSaveEprom1:
	.db "Save in " .db 0
StringSaveEprom2:
	.db "EEprom ?" .db 0
StringError:
	.db " Error  " .db 0

