;***********************************************************************
;
;	Intern Clock
;
;	
;
;	Author: Peter Rebesky
;
;	Date:	20.01.2013
;
;***********************************************************************
;
;	tested on ATmega8 with DCF77-Modul von www.pollin.de
;
;
;
;
;***********************************************************************
;
;	Definitions	extern
;
;***********************************************************************

;-----------------------------------
;	interupt for counts the seconds
countUP:
		in		saveSREG1, SREG		;save the status-register
		inc 	count				;increment counter
;rjmp d1 ;for debug and test modi only
		cpi 	count, div1s		;and is the second arrived?
		brcc 	c7					;no, return

c1:		out		SREG, saveSREG1		;restore the status-register
		reti

c7:		clr		count				;and restart count and test of definition between the rules
		inc 	S_second
		cpi 	S_second,60
		brcs 	c1
		inc 	S_minute
		cp 		S_minute,S_second
		brcs 	c2
d1:
		inc 	S_hour
		clr 	S_minute

		sbrs 	S_hour,4			;check of 24 hour: the 3th and 4th bit only
		rjmp 	c2
		sbrs 	S_hour,3
		rjmp 	c2
		inc 	S_day
		lds		S_hour, D_weekday	;misused the register to set Weekday
		inc		S_hour
		sbrs	S_hour, 3			;is the sunday arrived?
		rjmp	c6
		clr		S_hour
		inc		S_hour
c6:		sts		D_weekday, S_hour	;write weekday back
		clr 	S_hour
;check the days of month		
		push	temp1
		ldi		temp1, 31			;load the full amount of days
		mov		value, temp1
		lds		temp1, D_month		;input for the next function
		cpi		temp1, 2			;check if February
		brne	c4
		ldi		temp1, 28			;load 28 days
		sbrc	flags, 5			;check of leapyear and when then doesn't skip next commando
		ldi		temp1, 29			;load 29 days for a leapyear
		mov		value, temp1
		rjmp	c5
c4:		sbrc	temp1, 3			;if August(8), bit 3 is set
		subi	temp1, 1			;change the bit 0 by sub 1
		sbrs	temp1, 0			;when bit0 is set high then are 31 days, else 30 days
		dec		value				;sub by decrement one day from 31 to 30
c5:		cp		value, S_day		;if S_day higher than value -> no set, else set day on first of month
		pop		temp1
;---- end check ----
		brcc	c2					;the day isn't identically
		lds		S_day, D_month		;get the actual month by use the register of S_day
		inc		S_day				;now is month!!!
		sts		D_month, S_day		;write back the new month
		sbrs	S_day,3				;without using another register here in interupt it will be tested bits in byte for 12 or 13 (new year)
		rjmp	c3					;not bit 7, in conclusion it isn't december
		sbrs	S_day,2
		rjmp	c3
		sbrs	S_day,0
		rjmp	c3
		clr		S_day
		inc		S_day
		sts		D_month, S_day		;write back the new month
		lds		S_day, D_year		;get the actual year
		inc		S_day				;increment the year.... ohhh yeah it's Happy New Year now when this commando do its order!!!
		sts		D_year, S_day		;write back the new year
		
c3:		clr		S_day
		inc		S_day				;is necessary for the first day of month that is the value one.
c2:		clr 	S_second
		out		SREG, saveSREG1		;restore the status-register
		reti
;-----------------------------------
;-----------------------------------
;	prepairing timer1 for count the seconds
InitRTC:
		clr		temp1				;clear the TCT-register and move to it
		out 	TCNT1H,temp1 		;High Byte TC1 clear
		out 	TCNT1L,temp1 		;Low Byte TC1 clear
		ldi 	temp1,HIGH(divtc1) 	;then set the Compare Match, MSB first
		out 	OCR1AH,temp1
		ldi 	temp1,LOW(divtc1) 	;and Compare Match A, LSB last
		out 	OCR1AL,temp1
		ldi 	temp1,0b00001010 	;Clear on Comp Match A, Div 256
		out 	TCCR1B,temp1 		;Start Timer
		in 		temp1, TIMSK
		ori 	temp1,0b00010000 	;enable interupt TC1 CompA
		out 	TIMSK,temp1
		ret
;---end---
;-----------------------------------
;	initial all the values
InitValues:
		ldi 	temp1, div1s
		clr		count				;counter for RTC set to zero
		clr 	S_second		
		clr 	S_hour			
		clr 	S_minute
		clr		S_day
		ret
;---end---
;------------------------------------
;	convert from binary to (packed) BCD-Code in range of 0-99 (even to use in a clock-time)
;	input temp2-register, output temp2-register
;
bin2BCD:
		ldi		temp1, -1
_BCD1:	inc		temp1
		subi	temp2, 10
		brcc	_BCD1
		swap	temp1
		subi	temp2, -10
		or		temp2,temp1
		ret
;---end---
;------------------------------------
;	convert from (packed) BCD-Code to binary in range of 0-99 (even to use in a clock-time)
;	input temp2-register, output temp2-register
;
BCD2bin:
		mov 	temp1, temp2
		andi	temp2, 0x0f		;clear upper nipple
		andi	temp1, 0xf0		;clear lower nipple
		subi	temp2, 10	
_BIN1:	subi	temp2, -10		;add 10 while BCD unequal zero
		subi	temp1, 16
		brcc	_BIN1
		ret
;-----------------------------------
;	display a byte on LCD
;	input: temp2
ShowBCD:
		mov 	temp1, temp2
		swap	temp1
		andi	temp1, 0x0f
		ori		temp1, 0x30
		andi	temp2, 0x0f
		ori		temp2, 0x30
		rcall	LCD_Data
		mov		temp1,temp2
		rcall	LCD_Data
		ret

;******** synchronisierung *********

SynchronDCF:
		sbrc	flags,0				;skip if no error
		rjmp	_NO_SET
		ldi		temp2,1
		tst		DCFsecond			;skip if DCF-second unequal zero
		brne	_NO_SET
;		tst		S_second
;		breq	_NO_SET				;if S_second 1 already? than no-set
		cli
		ldi 	S_second,1
		clr 	count
		clr 	temp1
		ldi 	temp1,0b00001010 	;Clear on Comp Match A, Div 8
		out 	TCCR1B,temp1 		;Start Timer new
		sei
;***********************************
		lds		temp2,DCFminute
		rcall	BCD2bin
		mov		S_minute,temp2

		lds		temp2,DCFhour
		rcall	BCD2bin
		mov		S_hour,temp2

		lds		temp2,DCFday
		rcall	BCD2bin
		mov		S_day,temp2

		lds		temp2,DCFmonth
		rcall	BCD2bin
		sts		D_month,temp2

		lds		temp2,DCFweekday
		sts		D_weekday,temp2

		lds		temp2,DCFyear
		rcall	BCD2bin
		sts		D_year,temp2

		lds		temp2,DCFmez
		swap	temp2
		andi	temp2, 0x0f			;clear upper nipple because no validly information 
		sts		D_mez,temp2
		cbr		flags, 128			;clear the start-marker to indicate of the first time setting

_NO_SET:
		ret
;---end---
;***********************************
;
DisplayTime:
		mov 	temp2,S_hour
		rcall 	bin2BCD
;mov temp2,MenuCount
		rcall 	ShowBCD
		ldi		temp1, 0x3a
		rcall 	LCD_Data

		mov 	temp2,S_minute
		rcall 	bin2BCD
;mov temp2,MenuSetCount
		rcall 	ShowBCD
		ldi		temp1, 0x3a
		rcall 	LCD_Data
	
		mov 	temp2,S_second
		rcall 	bin2BCD
		rcall 	ShowBCD
		ret
;---end---
;******************************************************
;
DisplayDate:
		mov  	temp2,S_day
		rcall 	bin2BCD
		rcall 	ShowBCD
		ldi 	temp1,0x2e
		rcall 	LCD_Data
		lds 	temp2,D_month
		rcall 	bin2BCD
		rcall 	ShowBCD
		mov 	temp1,D_Info		;is stored a space when no Info occured else there is a info-character -> it isn't in used in this version!!! only a space
		sbrc 	flags,0				;if no ERROR by DCF77 show a space else show a /
		ldi 	temp1,0x2f			;Display a / between Date and Weekday if there an error of DCF77 identified
		rcall 	LCD_Data
;******************************************************
;
;	Display the Weekday like Mo, Di, Mi, Do, Fr, Sa, So, and for Error -> ER
;
DisplayWeekday:
		lds  	temp2, D_weekday		; 1= Mo, 2 = Di... 7 = So
;******************************************************
;	case / break - routine for decision of display the day of the week in range of 0 til 6
		dec 	temp2
		cpi 	temp2, 7
		brcc 	_NOT_IN_RANGE		;out of range (auerhalb des Bereiches)
		ldi 	ZL, low(Weekdays)
		ldi 	ZH, high(Weekdays)
		lsl 	temp2				;multiplied by two because there are two commandos in the jump-table
		add 	ZL,temp2			;variable addieren
		clr 	temp2
		adc 	ZH,temp2			;16 bit add include the carry-flag (important)
		ijmp
Weekdays:
		ldi 	XL,Montag		;case: 0
		rjmp 	_nextEnd
		ldi 	XL,Dienstag		;case: 1
		rjmp 	_nextEnd
		ldi 	XL,Mittwoch		;case: 2
		rjmp 	_nextEnd	
		ldi 	XL,Donnerstag	;case: 3
		rjmp 	_nextEnd
		ldi 	XL,Freitag		;case: 4
		rjmp 	_nextEnd
		ldi 	XL,Sonnabend	;case: 5
		rjmp 	_nextEnd
		ldi 	XL,Sonntag		;case: 6
		rjmp 	_nextEnd
_NOT_IN_RANGE: 					;case: default (ERROR)
		ldi 	XL,ERROR
_nextEnd:						;break
		ldi		XH,0
		rcall 	Show_String
		ret
;---end---
;******************************************************
;
;	Check the year if it a leapyear an set the bit 5 in flags-register
;	in: D_year = year between 0 and 39
;	out: bit5 in flagregister -> is in used by interupt-function to indicate if 29 or 28 days of february
;
CheckLeapyear:
		lds		data, D_year
		andi	data, 0b00000011	;check the lower two bits -> if they zero than is a leapyear
		tst		data
		breq	_leapyear
		cbr		flags, 32			;clear bit5, no leapyear
		ret
_leapyear:
		sbr		flags, 32			;set bit5 for leapyear	
		ret
;--- end ---
