;***********************************************************************
;
;	LCD-library for the LCD-Modul C0802-04
;
;	distrubied by www.pollin.de for low cost of 0,95
;
;	Author: Peter Rebesky
;
;	Date:	20.01.2013
;
;***********************************************************************
;
;	tested on ATtiny2313 and ATmega8
;	
;
;						LCD		Funktion		ATmega8
;						----------------------------------
;	LCD-connections: 	1		GND				GND
;						2		VDD (+5V)		Vcc
;						3		Vo (pull down over 820 Ohm)
;						4		RS				
;						5		R/W (GND)		GND
;						6		E (enable)		
;						7		D4				
;						8		D5				
;						9		D6				
;						10		D7				
;
;***********************************************************************
;
;	Definitions	are extern
;
;***********************************************************************
;
;	every funktion destroyed the register temp1, temp2 and r4 !!!
;	r4 is in use of delay -> it must be replaced if it's necessary
;
InitLCD:
		clr		LCD_C_Point
		inc		LCD_C_Point

		ldi		temp1, 0b00111111			;make bit 0 until 6 for output (connection: D4,D5,D6,D7,E, RS)
		out		LCD_Init_Port, temp1

		rcall 	delay_max					;initial-time to wait

		ldi		temp1, 0b00000011			;code for 8-bit interface mode and send it three times (it's described for init in the data-sheet of H44780U)
		out 	LCD_Data_Port, temp1

		rcall	delay_5ms
		sbi		LCD_Data_Port, PIN_LCD_E	;first time, impuls of enable
		rcall 	delay_5ms
		cbi 	LCD_Data_Port, PIN_LCD_E
		rcall 	delay_5ms
		sbi		LCD_Data_Port, PIN_LCD_E	;second time
		rcall 	delay_5ms
		cbi 	LCD_Data_Port, PIN_LCD_E
		rcall 	delay_5ms
		sbi		LCD_Data_Port, PIN_LCD_E	;thirdth time
		rcall 	delay_5ms
		cbi 	LCD_Data_Port, PIN_LCD_E
		rcall 	delay_5ms



		ldi 	temp1, 0b00000010			;set 4-bit interface mode and wait
		out		LCD_Data_Port, temp1
		sbi		LCD_Data_Port, PIN_LCD_E
		rcall 	delay_5ms
		cbi 	LCD_Data_Port, PIN_LCD_E
		rcall 	delay_5ms

		ldi 	temp1, 0b00101000			;zwei zeilig und 5*8 Punktdarstellung
		rcall 	LCD_Command
		rcall 	delay_5ms					;add 2ms for wait of LCD (it must be 4,1ms)

		ldi		temp1, 0b00000110			;coursor goes to right and no shift
		rcall 	LCD_Command

		rcall 	LCD_Clear
		rcall 	LCD_DisplayOn
;		rcall 	LCD_CoursorOn				; if this function activated, then blinking coursor on display

		ret

;-----------------------------------
;	send a commando to LCD by two nibbles
;	in: temp1
LCD_Command:
		push 	temp1
		swap 	temp1
		rcall 	CommandNibble
		pop 	temp1
CommandNibble:
		andi 	temp1, 0x0f					;clear the upper 4 bits (bit 4 must be zero because command to LCD)
		out 	LCD_Data_Port, temp1
		sbi 	LCD_Data_Port, PIN_LCD_E	;enable impulse
		rcall 	Delay_2ms
		cbi		LCD_Data_Port, PIN_LCD_E
		rjmp 	Delay_200us
;--- end ---
;-----------------------------------
;	send a databyte to LCD by two nibbles
;	in: temp1 -> it must be the 8-bit ascii code
LCD_Data:
		inc 	LCD_C_Point
		push 	temp1
		swap 	temp1
		rcall 	DataNibble
		pop 	temp1
DataNibble:
		andi 	temp1, 0x0f					;clear the upper 4 bits (bit 4 must be zero because command to LCD)
		out 	LCD_Data_Port, temp1
		sbi		LCD_Data_Port, PIN_LCD_RS	;RS must be set for send a ascii byte
		sbi 	LCD_Data_Port, PIN_LCD_E	;enable impulse
		rcall 	Delay_2ms
		cbi		LCD_Data_Port, PIN_LCD_E
		rjmp 	Delay_200us
;--- end ---
;-----------------------------------
;	reset coursor
LCD_1RowHome:
		clr		LCD_C_Point					;clear the coursor-position-counter
		ldi 	temp1, 0b00000010			;set display and coursor to begin
		rjmp 	LCD_Command
;--- end ---
;-----------------------------------
;	reset coursor second row 
LCD_2RowHome:
		rcall 	LCD_1RowHome				;set display and coursor to begin and clear LCD_C_Point
		ldi 	temp1, 0b11000000			;set coursor at row two
		rjmp 	LCD_Command
;--- end ---
;-----------------------------------
;	LCD clear and coursor at home 
LCD_Clear:
		rcall 	LCD_1RowHome				;set display and coursor to begin and clear LCD_C_Point
		ldi 	temp1, 0b00000001			;clear display
		rjmp 	LCD_Command
;--- end ---
;-----------------------------------
;	LCD switch off (dark)
LCD_Off:
		ldi 	temp1, 0b00001000			;display off
		rjmp 	LCD_Command
;--- end ---
;-----------------------------------
;	LCD switch on (display shows characters)
LCD_CoursorOn:
		ldi 	temp1, 0b00001111			;display and coursor on (blink)
		rjmp 	LCD_Command
LCD_DisplayOn:
		ldi 	temp1, 0b00001100			;display on, coursor off
		rjmp 	LCD_Command
;--- end ---
;-----------------------------------
;	LCD string goto left if coursor at 8th position
;	in: temp1 -> char (ascii)
;	if the position of coursor at the end of line, this funktion shift the writen text to the left
;	until the position goes at home (maximal 40 characters, after then is the begin at line two)

LCD_Data_Run:
		mov 	temp2, LCD_C_Point
		subi 	temp2, 9
		brcs 	LCD_Run_Normal

		push 	temp1

		ldi		temp2,5						;take time to read when the LCD-text is scrolling
loop_run:
		rcall 	delay_max					;this funktion run ca. 100 ms, in this time goes the interupt only 
		dec 	temp2
		brne 	loop_run

		ldi		temp1, 0b00011000			;LCD-commando for shift the display to left
		rcall 	LCD_Command

		pop		temp1
LCD_Run_Normal:
		rjmp 	LCD_Data
;--- end ---

;-----------------------------------
;	Delay by xxx cycles multiplied by temp1
;
;	this funktion destroyed the r4 !!!
;
Delay_max:
		ldi		temp1,0xff
		rjmp	delay
Delay_200us:
		ldi 	temp1, 1 + fclock * 2 / 10240000
		rjmp 	delay
Delay_2ms:
		ldi 	temp1, 1 + fclock * 20 / 10240000
		rjmp 	delay
Delay_5ms:
		ldi 	temp1, 1 + fclock * 50 / 10240000
		rjmp 	delay
Delay_8ms:
		ldi 	temp1, 1 + fclock * 80 / 10240000
		rjmp 	delay
Delay_15ms:
		ldi 	temp1, 1 + fclock * 150 / 10240000
		rjmp 	delay
delay:										;Tw = 1024 cycle * temp1
		push	temp1	;2 cycle
		push 	temp2	;2 cycle
		WDR				;1 cycle					;watchdoc reset, if it's used
		clr		temp2	;1 cycle
delay1:	
		subi 	temp2, 1 ;1 cycle
		sbci 	temp1, 0 ;1 cycle
		brne 	delay1	;2 cycle if true, else one cycle
		pop 	temp2	;2 cycle
		pop 	temp1	;2 cycle
		ret
;-----------------------------------
