;***********************************************************************
;
;	RFM01S (RFM02S) -library for the Radio-Modul
;
;	distrubied by www.pollin.de 
;
;	Author: Peter Rebesky
;
;	Date:	04.02.2013
;
;***********************************************************************
;
;	tested on ATmega8 and ATtiny2313
;
;***********************************************************************
; ports and pins definitions, all declared extern
;
;Transmitter RFM02S
;------------------
;PD0 = SDO(nIRQ)
;PD1 = FSK
;PD2 = nSEL
;PC5 = SDI
;PC4 = SCK

;Reciver RFM01S
;------------------
;PD0 = SDO
;PD1 = nSEL
;INT0(PD2) = nIRQ
;PD3 = SDI
;PD4 = SCK
;
;
;	Definitions	extern
;
;***********************************************************************
;
;
;***********************************************************************

InitSendPort:
		cbi		DDRD,send_SDO

		sbi		DDRD,send_nSEL
		sbi		DDRD,send_FSK
		sbi		DDRC,send_SDI
		sbi		DDRC,send_SCK

		cbi		PORTD,send_nSEL
		sbi		PORTD,send_FSK
		cbi		PORTC,send_SCK
		cbi		PORTC,send_SDI
		ret

;***********************************************************************
;
;	Write Send-Commando, 16 bit
;	input: rCMD_low, rCMDhigh
CMD_SEND:
		cbi		PORTC,send_SCK		;set low nSEL and SCK
		cbi		PORTD,send_nSEL
		ldi		temp1,15
_CMD1:
		sbrc	rCMD_high,7			;is there low- or high-bit ? if low than skip next operation
		sbi		PORTC,send_SDI		;set to high if no skip before
		sbrs	rCMD_high,7
		cbi		PORTC,send_SDI		;set PIN SDO on low
		nop
		nop
		nop
		lsl		rCMD_low			;shift the bits trought the register 24 and 25 on position MSB
		rol		rCMD_high
		sbi		PORTC,send_SCK		;once time clock on CLK
		nop
		nop
		nop
		nop
		nop
		cbi		PORTC,send_SCK		;clock set low again
		nop
		nop
		nop
		subi	temp1,1				;are all bits (word) send?
		brcc	_CMD1

		sbi		PORTD,send_nSEL		;set high nSEL and low SCK
		sbi		PORTC,send_SDI
		ret

;***********************************************************************
;
InitRFMsend:
		rcall	InitSendPort
		rcall	delay_max			;ATENTION must be calculated new, it must 1600ms!

		ldi		rCMD_high,0x88		;868MHz Band?, +/- 120kHz
;		ldi		rCMD_high,0x90		;868MHz Band?, +/- 120kHz ->it doesn't works!!!
		ldi		rCMD_low,0x63		;0x9063
		rcall	CMD_SEND
			
		ldi		rCMD_high,0xA6		;write frequency in range of 96 til 3903
		ldi		rCMD_low,0x40		;real f=868MHz ((0x640)1600 * 0,0050MHz + 860MHz)
		rcall	CMD_SEND

		ldi		rCMD_high,0xD2		;Powermanagement clock-out off
		ldi		rCMD_low,0x02		;
		rcall	CMD_SEND

		ldi		rCMD_high,0xC4		;sleep-commando in  clocks after tx close
		ldi		rCMD_low,0x00		;0x40
		rcall	CMD_SEND	

		ldi		rCMD_high,0xC8		;set data-rate (bps) = 10 000 000 / 29 / (R+1)
		ldi		rCMD_low,0x23		; (0x47)9,6 kbps
		rcall	CMD_SEND	

		ldi		rCMD_high,0xC2		;low battery detection command
		ldi		rCMD_low,0xA0		;and enable TX-syn funktion
		rcall	CMD_SEND	

		ldi		rCMD_high,0xC0		;command close rf and switch the send-parts off
		ldi		rCMD_low,0x60
		rcall	CMD_SEND	

		ret
;************************************************************************
;	send 8 bits
;	in: data (r18)
;	destroy: data
RFMsend:
		push	temp1
		push	temp2
		ldi		temp1,7				;8 bits counter
_RFMsn:
		ldi		temp2,255			;load 255 times for wait of SDO (nIRQ)
_RFMs1:
		sbic	PIND,send_SDO		;wait for high impuls
		rjmp	_RFMs2				;is high than jump for wait of low
		subi	temp2,1				;check for time-out, its necessary if nIRQ doesn't change
		brcs	_RFMerror			;the pin doesn't changed the level
		rjmp	_RFMs1				;OK, no error, jump to begin
_RFMs2:
		sbis	PIND,send_SDO		;wait for low again
		rjmp	_RFMsl				;low, than jump to write bit on FSK pin
		subi	temp2,1				;check for time-out, its necessary if nIRQ doesn't change
		brcs	_RFMerror
		rjmp	_RFMs2
_RFMsl:
		sbrs	data,7
		cbi		PORTD,send_FSK		;clear bit if bit 7 cleared in data
		sbrc	data,7
		sbi		PORTD,send_FSK		;set bit if bit high in data
		lsl		data				;shift the next bit to MSB for transmit
		ldi		temp2,255			;load 255 times for wait of SDO (nIRQ)
_RFMs3:
		subi	temp1,1				;check if send bits at all
		brcs	_RFMready
		rjmp	_RFMs1
_RFMerror:
		ldi		data, -1			;error-code: -1
_RFMready:							;data is empty if send bits at all
		pop		temp2
		pop		temp1
		ret
;************************************************************************
;
;
;
RFM_TX_data:
		ldi		rCMD_high,0xc0
		ldi		rCMD_low,0x38		;enable and open TX
		rcall	CMD_SEND
		rcall	delay_8ms			;time for start the transmitter-parts (tsx+tsp=5,25ms)
		ldi		temp1,3
		cli							;clear interuptflag -> critical time operation
_TX_AA:
		ldi		data,0xaa			;3 times syncrontime with 0xaa
		rcall	RFMsend
		cpi		data,-1				;if returned an error -> then jump out
		breq	_TX_OUT
		subi	temp1,1
		brcc	_TX_AA

		ldi		data,0x2d			;the word (high byte) to indicate the the next byte are values
		rcall	RFMsend
		cpi		data,-1				;if returned an error -> then jump out
		breq	_TX_OUT
		ldi		data,0xd4			;the word (low byte) to indicate the the next byte are values
		rcall	RFMsend
		cpi		data,-1				;if returned an error -> then jump out
		breq	_TX_OUT
;first byte -> addresse
		ldi		data,0x01			;the first reciever (address 1)
		rcall	RFMsend
		cpi		data,-1				;if returned an error -> then jump out
		breq	_TX_OUT
;second byte -> commando
		ldi		data,0x10			;turn on for 5 minutes
		rcall	RFMsend
		cpi		data,-1				;if returned an error -> then jump out
		breq	_TX_OUT
;third byte -> hours
		mov		data, S_hour		;send the actual time of hour
		rcall	RFMsend
		cpi		data,-1				;if returned an error -> then jump out
		breq	_TX_OUT
;fourth byte -> minute
		mov		data, S_minute		;send the actual time of minute  
		rcall	RFMsend
		cpi		data,-1				;if returned an error -> then jump out
		breq	_TX_OUT
;fiveth byte -> day
		mov		data, S_day			;send the actual day of date 
		rcall	RFMsend
		cpi		data,-1				;if returned an error -> then jump out
		breq	_TX_OUT
;sixth byte -> month
		lds		data, D_month		;send the actual month of date
		rcall	RFMsend


_TX_OUT:
		ldi		temp1,3
_TX_CLOSE:
		ldi		data,0x00			;3 times 0x00 -> end of transmission 
		rcall	RFMsend
		subi	temp1,1
		brcc	_TX_CLOSE

		ldi		rCMD_high,0xC0		;command close rf and switch the send-parts off
		ldi		rCMD_low,0x20
		rcall	CMD_SEND	
		reti


;************************************************************************
	
