;***********************************************************************
;
;	SetMenu Structures
;
;	
;
;	Author: Peter Rebesky
;
;	Date:	15.03.2013
;
;***********************************************************************
;
;	
;
;
;
;
;***********************************************************************
;
;	Definitions	extern
;
;***********************************************************
;
;	select a menu through the count the menucounter by button (right or middle)
;	input: 
;	output: MenuCount, temp1 ok= 0, fault= -1 (0xff)	
;	
SelectMenu:	
		sbic	ButtonPort, ButtonRight
		rjmp	_SelectMenu1
		sbr		flags,2
		ret
_SelectMenu1:
		sbrs	flags,1						;button debouncing
		ret
		sbrs	MenuSetCount,7
		inc		MenuCount					;increase counter of main-menu
		sbrc	MenuSetCount,7
		inc		MenuSetCount				;increase counter of Set-Menu
		cbr		flags,2

		lds		temp1, TimerFlags			;start timer for wait of input
		ori		temp1, 0b00000100			;set flag of start timer2 -> is in use for automatic reset of menu for display the date at row two
		sts		TimerFlags, temp1
		clr		temp1
		sts		Timer2,temp1				;reset timer2 new of every push the button

		mov		temp1, MenuCount			;if reached the MenuItem
		cpi		temp1, MenuItem + 1
		brcs	_SelectMenu2
		clr		MenuCount					;begin new by 0
		clr		MenuSetCount
		ret
_SelectMenu2:
		sbrc	MenuSetCount,4				;maximal until bit 4 (16 entries) is set, then set to begin
		clr		MenuSetCount
		ret

;--- end ---
;***********************************************************
;
;	Showed the status on Display on row two and toggle it
;			
SwitchOn_Off:
		ldi		XH,high(StringOff)
		ldi		XL,low(StringOff)
		sbrs	flags, 6					;if turn automatic off / on
		ldi 	XL,low(StringOn)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		rjmp	_SwitchOn_OFF
		;action when pressed button
		sbr		flags,4
		ret
_SwitchOn_OFF:
		sbrs	flags,2						;button debouncing
		ret
		cbr		flags,4
		;action when left button
		sbrs	flags,6						;exor with bit 6
		rjmp	_Set_On_Off
		cbr		flags,64					
		ret
_Set_On_Off:
		sbr		flags,64
		ret

;--- end ---
;***********************************************************************
;
;	Display the Set-Mode on at row two
;			
Set_Mode:
		sbrc	MenuSetCount, 7
		rjmp	MenuBelow
		ldi		XH,high(StringSet)
		ldi 	XL,low(StringSet)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft		;if button pressed for ok?
		rjmp	_SetMode					;no then skip this operation
		;action if pressing left button
		sbr		flags,4
		ret

_SetMode:
		sbrs	flags,2						;button debouncing
		ret
		cbr		flags,4
		;action when left button
		ldi		temp1, 0x80
		mov		MenuSetCount, temp1			;set bit 7 in SetMenuCounter
		ret

MenuBelow:
		;here is the code during the Set-Menu
;******************************************************
;	case / break - routine for decision of display the menue in range of 0 till MenuSetItem (8)
		mov 	temp2, MenuSetCount
		cbr		temp2, 128			;clear bit7 what indicated if Set-menu is running -> it doesnt need to calculated the jmp adresse!
		cpi 	temp2, 12			;count of range 
		brcc 	_NOT_SET_MENU		;out of range (auerhalb des Bereiches)
		ldi 	ZL, low(_SetMenu)
		ldi 	ZH, high(_SetMenu)
		lsl 	temp2				;multiplied by two because there are two commandos in the jump-table
		add 	ZL,temp2			;variable addieren
		clr 	temp2
		adc 	ZH,temp2			;16 bit add include the carry-flag (important)
		ijmp
_SetMenu:
		rcall	SetTimeHour			;case: 0
		ret
		rcall	SetTimeMinute		;case: 1
		ret
		rcall	SetMonth			;case: 2
		ret
		rcall	SetDay				;case: 3
		ret
		rcall	SetYear				;case: 4
		ret
		rcall	SetWeekDay			;case: 5
		ret
		rcall	SetCloseSec			;case: 6
		ret
		rcall	SetOpenSec			;case: 7
		ret
		rcall	SetLightHour		;case: 8
		ret
		rcall	SetLightMinute		;case: 9
		ret
		rcall	SetMEZ_MESZ			;case: 10
		ret
		rcall	SaveEprom			;case: 11
		ret
_NOT_SET_MENU: 						;case: default (ERROR)
		clr		MenuSetCount		;if out of range then clear MenuSetCounter (zero) and jump to case 0
;		ldi 	XH,high(StringError)
;		ldi		XL,low(StringError)
;		rcall 	Show_String		
		ret
;--- end ---
;***********************************************************
;
;	Set the Time
;			
SetTimeHour:
		rcall 	LCD_1RowHome		;set the first position on LCD
		rcall 	DisplayTime			;show the time on LCD at first line by 8 character like 10:23:43

		rcall 	LCD_2RowHome		;next row on the display

		ldi		XH,high(StringSetH)
		ldi 	XL,low(StringSetH)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetHour
		;action when pressed button
		sbr		flags,4
		ret
_SetHour:
		sbrs	flags,2						;button debouncing
		ret
		cbr		flags,4
		;action when left button
		inc		S_hour
		ldi		temp1, 23					;load maximal count of hours
		cp		temp1, S_hour
		brcc	_SetRet
		clr		S_hour
		ret

;***********************************************************

SetTimeMinute:
		rcall 	LCD_1RowHome		;set the first position on LCD
		rcall 	DisplayTime			;show the time on LCD at first line by 8 character like 10:23:43

		rcall 	LCD_2RowHome		;next row on the display
		ldi		XH,high(StringSetM)
		ldi 	XL,low(StringSetM)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetMinute
		;action when pressed button
		sbr		flags,4
		ret
_SetMinute:
		sbrs	flags,2						;button debouncing
		ret
		cbr		flags,4
		;action when left button
		inc		S_minute
		clr		S_second
		ldi		temp1, 59					;load maximal count of minutes
		cp		temp1, S_minute
		brcc	_SetRet
		clr		S_minute
_SetRet:
		ret

;--- end ---
;***********************************************************
;
;	Set open seconds
SetOpenSec:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringRO)
		ldi 	XL,low(StringRO)
		rcall 	Show_String
		rcall	LCD_2RowHome
		ldi		XH,high(StringROP)
		ldi 	XL,low(StringROP)
		rcall 	Show_String
		lds		temp2, Time1
		rcall	Bin2BCD
		rcall	ShowBCD

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetOpenSec
		;action when press button
		sbr		flags,4						;for button debouncing
		ret
_SetOpenSec:
		sbrs	flags,2						;for button debouncing
		ret
		cbr		flags,4						;for button debouncing
		;action when left button
		lds		temp1, Time1
		inc		temp1
		cpi		temp1, 120
		brcs	_SetOpenRet
		clr		temp1
_SetOpenRet:
		sts		Time1, temp1
		ret
;
;--- end ---	
;***********************************************************
;
;	Set close seconds
;
SetCloseSec:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringRO)
		ldi 	XL,low(StringRO)
		rcall 	Show_String
		rcall	LCD_2RowHome
		ldi		XH,high(StringRCL)
		ldi 	XL,low(StringRCL)
		rcall 	Show_String
		lds		temp2, Time0
		rcall	Bin2BCD
		rcall	ShowBCD

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetCloseSec
		;action when press button
		sbr		flags,4						;for button debouncing
		ret
_SetCloseSec:
		sbrs	flags,2						;for button debouncing
		ret
		cbr		flags,4						;for button debouncing
		;action when left button
		lds		temp1, Time0
		inc		temp1
		cpi		temp1, 120
		brcs	_SetCloseRet
		clr		temp1
_SetCloseRet:
		sts		Time0, temp1
		ret
;--- end ---	
;***********************************************************
;
;	Set month
;
SetMonth:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringDisplayMonth)
		ldi 	XL,low(StringDisplayMonth)
		rcall 	Show_String
		lds		temp2, D_month
		rcall	Bin2BCD
		rcall	ShowBCD

		rcall	LCD_2RowHome
		ldi		XH,high(StringSetMD)
		ldi 	XL,low(StringSetMD)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetMonth
		;action when press button
		sbr		flags,4						;for button debouncing
		ret
_SetMonth:
		sbrs	flags,2						;for button debouncing
		ret
		cbr		flags,4						;for button debouncing
		;action when left button
		lds		temp2, D_month
		inc		temp2
		ldi		temp1, 12					;load maximal count of months
		cp		temp1, temp2
		brcc	_SetMRet
		clr		temp2
_SetMRet:
		sts		D_month, temp2
		ret
;--- end ---	
;***********************************************************
;
;	Set day
;
SetDay:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringDisplayDay)
		ldi 	XL,low(StringDisplayDay)
		rcall 	Show_String
		mov		temp2, S_day
		rcall	Bin2BCD
		rcall	ShowBCD

		rcall	LCD_2RowHome
		ldi		XH,high(StringSetMD)
		ldi 	XL,low(StringSetMD)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetDay
		;action when press button
		sbr		flags,4						;for button debouncing
		ret
_SetDay:
		sbrs	flags,2						;for button debouncing
		ret
		cbr		flags,4						;for button debouncing
		;action when left button
		inc		S_day
		ldi		temp1, 31					;load maximal count of days per month
		cp		temp1, S_day
		brcc	_SetDRet
		clr		S_day
_SetDRet:
		ret
;--- end ---
;***********************************************************
;
;	Set year
;
SetYear:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringDisplayYear)
		ldi 	XL,low(StringDisplayYear)
		rcall 	Show_String

		rcall	LCD_2RowHome
		ldi		temp1, 32 					;load space
		rcall	LCD_Data
		ldi		temp1, 63 					;load "?"
		rcall	LCD_Data
		ldi		temp1, 32 					;load space
		rcall	LCD_Data
		ldi		temp1, 32 					;load space
		rcall	LCD_Data
		ldi		temp2, 20					;load 20 for two thousand (millennium)
		rcall	Bin2BCD
		rcall	ShowBCD
		lds		temp2, D_year
		rcall	Bin2BCD
		rcall	ShowBCD

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetYear
		;action when press button
		sbr		flags,4						;for button debouncing
		ret
_SetYear:
		sbrs	flags,2						;for button debouncing
		ret
		cbr		flags,4						;for button debouncing
		;action when left button
		lds		temp2, D_year
		inc		temp2
		ldi		temp1, 39					;load maximal count of years (in this case maximal 2039)
		cp		temp1, temp2
		brcc	_SetYRet
		clr		temp2
_SetYRet:
		sts		D_year, temp2
		ret
;--- end ---	
;***********************************************************
;
;	Set weekday
;
SetWeekday:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringDisplayWeekday)
		ldi 	XL,low(StringDisplayWeekday)
		rcall 	Show_String

		rcall	LCD_2RowHome
		ldi		XH,high(StringSetWeekday)
		ldi 	XL,low(StringSetWeekday)
		rcall 	Show_String
		rcall	DisplayWeekday

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetWeekday
		;action when press button
		sbr		flags,4						;for button debouncing
		ret
_SetWeekday:
		sbrs	flags,2						;for button debouncing
		ret
		cbr		flags,4						;for button debouncing
		;action when left button
		lds		temp2, D_weekday
		inc		temp2
		ldi		temp1, 7					;load maximal count of weekdays
		cp		temp1, temp2
		brcc	_SetWRet
		ldi		temp2, 1
_SetWRet:
		sts		D_Weekday, temp2
		ret
;--- end ---	
;***********************************************************
;
;	Set light off time of hour
;
SetLightHour:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringLightClock)
		ldi 	XL,low(StringLightClock)
		rcall 	Show_String

		rcall	LCD_2RowHome
		ldi		XH,high(StringLightHour)
		ldi 	XL,low(StringLightHour)
		rcall 	Show_String
		lds		temp2, L_hour
		rcall	Bin2BCD
		rcall	ShowBCD

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetLightHour
		;action when pressed button
		sbr		flags,4
		ret
_SetLightHour:
		sbrs	flags,2						;button debouncing
		ret
		cbr		flags,4
		;action when left button
		lds		temp2, L_hour
		inc		temp2
		ldi		temp1, 23					;load maximal count of minutes
		cp		temp1, temp2
		brcc	_SetLightHRet
		clr		temp2
_SetLightHRet:
		sts		L_hour, temp2
		ret
;
;--- end ---	
;***********************************************************
;
;	Set light off time of minute
;
SetLightMinute:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringLightClock)
		ldi 	XL,low(StringLightClock)
		rcall 	Show_String

		rcall	LCD_2RowHome
		ldi		XH,high(StringLightMinute)
		ldi 	XL,low(StringLightMinute)
		rcall 	Show_String
		lds		temp2, L_minute
		rcall	Bin2BCD
		rcall	ShowBCD

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetLightMinute
		;action when pressed button
		sbr		flags,4
		ret
_SetLightMinute:
		sbrs	flags,2						;button debouncing
		ret
		cbr		flags,4
		;action when left button
		lds		temp2, L_minute
		inc		temp2
		ldi		temp1, 59					;load maximal count of minutes
		cp		temp1, temp2
		brcc	_SetLightMRet
		clr		temp2
_SetLightMRet:
		sts		L_minute, temp2
		ret
;
;--- end ---	
;***********************************************************
;
;	change mez-mesz
;
SetMEZ_MESZ:
		rcall 	LCD_1RowHome		;set the first position on LCD
		lds		temp2, D_mez
		sbrc	temp2, 1			;if set it isn't summertime
		rjmp	_MESZ
		ldi		XH,high(StringMEZ)
		ldi 	XL,low(StringMEZ)
		rjmp	_Display
_MESZ:
		ldi		XH,high(StringMESZ)
		ldi 	XL,low(StringMESZ)
_Display:
		rcall 	Show_String

		rcall	LCD_2RowHome
		ldi		XH,high(StringChange)
		ldi 	XL,low(StringChange)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		rjmp	_SetMEZ
		;action when pressed button
		sbr		flags,4						;for button debouncing
		ret
_SetMEZ:
		sbrs	flags,2						;for button debouncing
		ret
		cbr		flags,4						;for button debouncing
		;action when left button
		lds		temp2, D_mez
		ldi		temp1, 0x06
		eor		temp2, temp1
		sts		D_mez, temp2
		ret
;
;--- end ---	
;**********************************************************
;
;	Save the values into EEprom
;	
SaveEprom:
		rcall 	LCD_1RowHome		;set the first position on LCD
		ldi		XH,high(StringSaveEprom1)
		ldi 	XL,low(StringSaveEprom1)
		rcall 	Show_String

		rcall	LCD_2RowHome
		ldi		XH,high(StringSaveEprom2)
		ldi 	XL,low(StringSaveEprom2)
		rcall 	Show_String

		sbic	ButtonPort, ButtonLeft
		rjmp	_SaveEprom
		;action when press button
		sbr		flags,4						;for button debouncing
		ret
_SaveEprom:
		sbrs	flags,2						;for button debouncing
		ret
		cbr		flags,4						;for button debouncing
		;action when left button
		ldi		XH, high(LightOff)			;turn off time for the light
		ldi		XL, low(LightOff)
		lds		data, L_minute
		rcall	WriteByteEprom
		adiw	XH:XL,1
		lds		data, L_hour
		rcall	WriteByteEprom

		ldi		XH, high(CloseTime)			;close time in seconds to turn on the motor
		ldi		XL, low(CloseTime)
		lds		data, Time0
		rcall	WriteByteEprom

		ldi		XH, high(OpenTime)			;open time in seconds to turn on the motor
		ldi		XL, low(OpenTime)
		lds		data, Time1
		rcall	WriteByteEprom
		
		clr		MenuSetCount				;clear menu-counters for return of regular display
		clr		MenuCount
		ret
;---end---
;**********************************************************
;
;	Write a byte (data) by adress XH:XL into EEPROM
;	in: XH:XL = adress; data = byte
;
WriteByteEprom:
		sbic	EECR, EEWE
		rjmp	WriteByteEprom	;if here are a faulure than will the watchdoc do a reset
		out		EEARH, XH
		out		EEARL, XL
		out		EEDR, data
		cli						;avoid an interupt because it isn't be allowed to write into the RAM during an Eprom-writing!		
		sbi		EECR,EEMWE
		sbi		EECR,EEWE		;order to write the byte into eeprom
		reti					;turn the interupt on

;--- end ---
;**********************************************************
